//
// Copyright (C) 2013..14 Falk Mueller
//
// mailbox@anfatec.de
// Falk Mueller
// Melanchthonstr. 28
// Oelsnitz
// D-08606
// Germany



library LED;

{ Important note about DLL memory management: ShareMem must be the
  first unit in your library's USES clause AND your project's (select
  Project-View Source) USES clause if your DLL exports any procedures or
  functions that pass strings as parameters or function results. This
  applies to all strings passed to and from your DLL--even those that
  are nested in records and classes. ShareMem is the interface unit to
  the BORLNDMM.DLL shared memory manager, which must be deployed along
  with your DLL. To avoid using BORLNDMM.DLL, pass string information
  using PChar or ShortString parameters. }

uses
  SysUtils,
  Classes,
  Inifiles,
  Dialogs,
  Registry,
  Windows,
  Forms,
  Com in 'Com.pas',
  Main in 'Main.pas' {MainForm};

type
  TMyThread = class(TThread)
  private
  protected
    procedure Execute; override;
  end;

type
  tpDouble = ^double;
  tpInteger = ^integer;


{$R *.RES}
  
var MyThread : TMyThread;

///////////////////////////////////////
///////////////////////////////////////
///////////////////////////////////////

// for dll messages
procedure TMyThread.Execute;
var
  Msg : TMsg;
begin
  Application.CreateForm(TMainForm, MainForm);
  MainForm.Timer1Timer(MainForm);
  MainForm.Timer1.Enabled:=true;

  while (not Terminated) do begin
    if integer(GetMessage(Msg,0,0,0))=-1 then
    //if integer(PeekMessage(Msg,0,0,0,PM_REMOVE))=-1 then
      break // close
    else begin
      TranslateMessage(Msg);
      DispatchMessage(Msg);
    end;
    sleep(1);
  end;

  MainForm.Free;
  Application.Terminate;
  sleep(10);
  Main.StoppedApp:=true;
end;

////////////////////////////////////////////////////////////////////////////////
procedure DllMain (Reason : DWord);
var i : integer;
begin
  case Reason of
  DLL_PROCESS_ATTACH : begin
    DisableThreadLibraryCalls(HInstance);
    MyThread := TMyThread.Create(false);
    MyThread.Priority := tpNormal;
    MyThread.Resume;
  end;

  DLL_THREAD_ATTACH : begin
    sleep(1);
  end;
  DLL_THREAD_DETACH : begin
    sleep(1);
  end;
  DLL_PROCESS_DETACH : begin

    Main.StoppedApp:=false;
    MyThread.Terminate;

    i:=0;
    repeat
      sleep(100);
      inc(i);
    // with timeout
    until (Main.StoppedApp) or (i>20); // wait for MyThread.Execute

    // to be sure
    if Cid<>INVALID_HANDLE_VALUE then begin
      CloseHandle(Cid);
    end;

  end;
  end;
end;

///////////////////////////////////////////////////////////////////
// exported functions


// LED_Nr: 1..40
// Brightness: 0..100
procedure LED_SetLEDWithBrightness(LED_Nr : tpInteger; Brightness: tpdouble); cdecl;
begin
  MainForm.SelectLampByUser(LED_Nr^, Brightness^);
end;

procedure LED_GetWaveLength(LED_Nr: tpInteger; Output: PChar);  cdecl;
var
  i : integer;
  nm: String;
begin
  nm := Main.LEDIni.ReadString('LED' + IntToStr(LED_Nr^), 'Wavelength', '0');
  for i:=1 to length(nm) do  Output[i-1] := nm[i];
  Output[length(nm)]:=#0;
end;

procedure LED_GetMaxPower(LED_Nr: tpInteger; MaxPower: tpDouble); cdecl;
begin
  MaxPower^  := Main.LEDIni.ReadFloat('LED' + IntToStr(LED_Nr^), 'Brightness', 0);
end;

procedure LED_GetAUXin(Value: tpDouble); cdecl;
begin
  Value^:=MainForm.AUXinValue;
  // is done in Timer WriteChar('v'); //Rainbow source, please Check Auxin Voltage
end;

// try to set LED with a fixed power in mW
// in return PowerLevel is used level
procedure LED_SetPowerLevel(LED_Nr: tpInteger; PowerLevel: tpDouble);  cdecl;
var
  mW: Double;
  Bright: Real; // in %
begin
  // read max mW from *.ini
  mW := Main.LEDIni.ReadFloat('LED' + IntToStr(LED_Nr^), 'Brightness', 0);

  Bright := 0; // default off
  if mW <> 0 then begin
    Bright := powerLevel^ / mW * 100; //in %
    if Bright < 0 then Bright := 0;
    if Bright > 100 then Bright := 100;
  end;
  powerLevel^ := Bright/100*mW; // in mW

  MainForm.SelectLampByUser(LED_Nr^, Bright);
end;

////////////////////////////////////////////////////////////////////////////////

exports
  // SCilab need name_
  // LabView needs _name
  // so we use _name_

  LED_SetLEDWithBrightness    name '_LEDSetLamp_',
  LED_SetPowerLevel           name '_LEDSetPowerlevel_',
  LED_GetWaveLength           name '_LEDGetWavelength_',
  LED_GetMaxPower             name '_LEDGetMaxPower_',
  LED_GetAUXin                name '_LEDGetAUXin_';

begin
  DllProc:=@DllMain; // don't forget to patch system.dcu in delphi 6
  DllMain(DLL_PROCESS_ATTACH);
end.
