unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls,
  IniFiles,
  Registry;

type
  TMainForm = class(TForm)
    Label1: TLabel;
    Timer1: TTimer;
    procedure FormShow(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    SerNr    : integer; // Nr. from *.ini
    SerNrDev : integer; // Nr. from Device
    MotorHomeState : integer; // 0 or 1 or 2
    IsHoming : boolean;
    ErrorInIniFile : integer;
    CurrLED : integer;
    CurrBrightness : real;
    procedure CheckUserMessage;
    procedure ProcessMessageFromRainbowSource (ToRead : integer);
    procedure ShowVolt (S: String);
    procedure CheckSerNrAndStepper(s : String);
    procedure DoHome;  // Set Motor to Home
  public
    PathToIniFile : String;
    AUXinValue : real;
    procedure SelectLampByUser(LED_Nr: Integer; Brightness : real);
    procedure SetBrightness(Bright : real);
  end;

var
  MainForm: TMainForm;
  StoppedApp : boolean; // start stop dll
  LEDIni: TIniFile;


function GivePathToIniFromRegistry (var Path : String) : boolean;

implementation

{$R *.dfm}

uses
  Com;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  Com.Cid := INVALID_HANDLE_VALUE;

  //Check registry
  if not GivePathToIniFromRegistry (PathToIniFile) then exit;

  LEDIni := TIniFile.Create(PathToIniFile+'LEDs.ini');
  SerNr:=LEDIni.Readinteger('Main', 'SerNr', -1);
  // first stepper command checks SerNr later

  CurrLED := LEDIni.ReadInteger('Initialize', 'ScrollbarLamp', 0);
  CurrBrightness :=LEDIni.ReadInteger('Initialize', 'ScrollbarBright', 10);

  IsHoming := false;
  ErrorInIniFile := 0;
end;


procedure TMainForm.FormShow(Sender: TObject);
begin
  CheckUserMessage;
end;

procedure TMainForm.CheckUserMessage;
begin
  Label1.Caption:='';
  if Com.Cid = INVALID_HANDLE_VALUE then begin
    Label1.Caption:=
          'Cant open com'+IntToStr(Com.UsedPort)+'!'+#13+
          'Check "Com.ini" and USB-Port!';
  end else
  if SerNr<>SerNrDev then begin
    Label1.Caption:=
      'SerNr. in "LEDs.ini" does not'+#13+
      'match Device!';
  end else
  if MotorHomeState<>2 then begin
    Label1.Caption:=
      'Rainbow Source is Homing.';
  end else
  if  ErrorInIniFile>0 then begin
    Label1.Caption:=
      'Error in LEDs.ini!';
  end;

  if Label1.Caption='' then begin
    Visible:=false;
  end else begin
    Show;
  end;
end;

procedure TMainForm.Timer1Timer(Sender: TObject);
var
  InQue, OutQue: longint;
begin
  if Com.CID<>INVALID_HANDLE_VALUE then begin
    DataInBuffer(Cid,InQue,OutQue);
    if (InQue>0) then   // data ready to read
      ProcessMessageFromRainbowSource (InQue);
  end else begin
    OpenComm();
    if Com.Cid=INVALID_HANDLE_VALUE then begin
      Show;
      exit;
    end;
    SelectLampByUser(CurrLED, CurrBrightness);// now CID is OK
  end;

  CheckUserMessage;
  if IsHoming then exit; // wait till home has finished
  if not Timer1.Enabled then exit; // for Stop com port

  WriteChar('v'); //CheckVoltage Aux in
end;

procedure TMainForm.ProcessMessageFromRainbowSource (ToRead : integer);
const
  TempBufSize = 2048;
var
  Index : integer;
  ch : char;
  ReadBuffer : array[0..TempBufSize] of char;
  S : String;
  HasRead : cardinal;

function ReadTillEnd : String;
const number='0123456789';
var S : string;
    ch : char;
begin
  S:='';
  repeat
    ch:=ReadBuffer[Index];
    if ch=#0 then break;
    inc(Index);
    if Pos(ch,number)=0 then begin // not a number?
      dec(Index); // give last char back
      break;
    end;
    S:=S+ch;
  until false;
  Result:=S;
end;

begin
  if (ToRead>TempBufSize) then ToRead:=TempBufSize;
  ReadFile(Cid, ReadBuffer, ToRead, HasRead,nil);
  ReadBuffer[HasRead]:=#0;

  Index:=0;
  repeat
    ch:=ReadBuffer[Index];
    inc(Index);
    if ch='h' then begin // "reached home"
      IsHoming :=false;
      MotorHomeState := 2; // so is homed
      SelectLampByUser(CurrLED, CurrBrightness);//
      Continue;
    end;
    if ch='v' then begin // check aux voltage
      S:=ReadTillEnd;
      ShowVolt(S);
      Continue;
    end;
    if ch='B' then begin // Response from photo diode
      S:=ReadTillEnd;
      //ShowBrightness(S);
      Continue;
    end;
    if ch='M' then begin // Response from Stepper 'm'
      S:=ReadTillEnd;
      CheckSerNrAndStepper(S);
      Continue;
    end;

  until ch=#0; // last char
end;

procedure TMainForm.ShowVolt (S: String);
var
  voltage: Integer;
  Offset, Scale : real;
begin
  try
    voltage := StrToInt(S);
    Offset := LEDIni.ReadFloat('Initialize', 'AUXOffset', 519);
    Scale  := LEDIni.ReadFloat('Initialize', 'AUXScale', 21.1);
    AUXinValue :=(Voltage-Offset)/1024*Scale;
  except
  end;
end;


procedure TMainForm.CheckSerNrAndStepper(S : String);
var
  Data : integer;
  // now global MotorHomeState : integer;
begin
  try
    Data:=StrToInt(S);
  except
    exit;
  end;
  SerNrDev:=Data shr 2;

  // 0=device powerd up
  // 1=search home
  // 2=found home
  MotorHomeState:=Data and 3;

  if SerNr<>SerNrDev then begin //*.ini SerNr <> DevSerNr
    Show;
    Exit;
  end;
  if (not IsHoming) and (MotorHomeState<2) then begin
    IsHoming := True;
    Show;
    DoHome;//need home!!!
  end;  

end;

procedure TMainForm.DoHome;  // Set Motor to Home
begin
  //ver. 12c Write16BitInt('l', 0);    // disable lamps
  SetBrightness(-0.001); // save power

  Write16BitInt('m', 400); // 400 steps = home
end;


///////////////////////////////////////

//////////////////////////////////////////////////////////

function GivePathToIniFromRegistry (var Path : String) : boolean;
var
  IORegistry : TRegistry;
  SerNr : integer;
  MainKey, S : String;
begin
  Result :=false; // no Entry for Rainbow in Registry
  Path :='';

  IORegistry:=TRegistry.Create;
  IORegistry.RootKey:=HKEY_LOCAL_MACHINE;

  try
    MainKey:='Software\Anfatec\Rainbow';
    // *.exe was never executed
    if (not IORegistry.KeyExists(MainKey))then begin
      MessageDlg('Please run "LEDLamp.exe" to install Rainbow Source first!', mtInformation,
      [mbOk], 0);
      halt(0);
    end;

    // try my best to find SerNr
    for SerNr:=1 to 100 do begin
      S:=MainKey+'\'+IntToStr(SerNr);
      if (not IORegistry.KeyExists(S)) then continue; // not my SerNr

      IORegistry.OpenKey (S, false); // false = don't create
      Path:=IORegistry.ReadString('PathToIni');
      IORegistry.CloseKey;
      if (length(Path)>2) and (path[1]='"') then
        Path:=copy(Path,2, length(path)-2);

      // now we have a Path but is it still there?
      if not FileExists(Path+'LEDs.ini') then begin
        MessageDlg(
          '"'+Path+'LEDs.ini" not found!'+#13+
          'Please rerun "LEDLamp.exe" to install Rainbow Source!', mtInformation,
          [mbOk], 0);
        Halt(0);
      end;

      Result:=true;
    end;
  finally
    IORegistry.Free;
  end;
end;

procedure TMainForm.SelectLampByUser(LED_Nr: Integer; Brightness : real);
var
  step: Longint;
begin
  SetBrightness(-0.001); // for safety of LEDs and save power

  step := LEDIni.ReadInteger('LED' + IntToStr(LED_Nr), 'Step', 0);
  Write16BitInt('m', Step);
  sleep(200); // wait till dark

  CurrLED:=LED_Nr;
  if Brightness>0 then begin
    Write16BitInt('l', currLED);// Ver. 1.2c
    SetBrightness(Brightness);
  end else
    Write16BitInt('l', 0);    //TurnLampsOff;
end;

procedure TMainForm.SetBrightness(Bright : real);
var
  MinCurrLED, MaxCurrLED : Real;
  MinCurrentDev, MaxCurrentDev : Real;
  DacVal : integer;
  mAmpere : real; // in mA !
  LEDName : String;
begin
  LEDName :=IntToStr(CurrLED);
  try //max. current for this LED
    MaxCurrLED := StrToFloat(LEDIni.ReadString(LEDName, 'Current', '10')); //max. current
  except
    inc(ErrorInIniFile);
    MaxCurrLED:=0;
    if Bright<>0 then exit; // allow 0%
  end;

  try //min. current for this LED
    MinCurrLED := StrToFloat(LEDIni.ReadString(LEDName, 'Start', '0')); //min. current
  except
    inc(ErrorInIniFile);
    MinCurrLED:=0;
    if Bright<>0 then exit; // allow 0%
  end;

  // this USB Device
  MinCurrentDev := LEDIni.ReadFloat('Initialize', 'MinCurrent', 9.18);
  MaxCurrentDev := LEDIni.ReadFloat('Initialize', 'MaxCurrent', 100);

  mAmpere := MinCurrLED+(MaxCurrLED-MinCurrLED)/100 * Bright;
  DACVal := round((mAmpere-MinCurrentDev) / MaxCurrentDev * $3ff);
  if DACVal>$3ff then DaCVal:=$3ff; // more than max current of device
  if DACVal<0 then DACVal:=0;// less than no current
  if Bright<0 then DACVal:=0;// ver. 1.2c

  Write16BitInt('b', DACVal);
  //Ver. 1.2c if DACVal>0 then Write16BitInt('l', currLED);

  CurrBrightness:=Bright;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  LEDIni.WriteInteger('Initialize', 'ScrollbarLamp', CurrLED);
  LEDIni.WriteInteger('Initialize', 'ScrollbarBright', round(CurrBrightness));

  //Write16BitInt('l', 0);  // disable lamps
  SetBrightness(-0.001);

  CloseCom;
  LEDIni.Free;
end;

end.
