unit Test;

{ $define static} // static or dynamic link to dll
{$undef static} // static or dynamic link to dll

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls;

type
  tpDouble = ^double;
  tpInteger = ^integer;

type
  TForm1 = class(TForm)
    ScrollBar1: TScrollBar;
    Label1: TLabel;
    Button1: TButton;
    Button2: TButton;
    Label2: TLabel;
    Label3: TLabel;
    ScrollBar2: TScrollBar;
    Label4: TLabel;
    Label5: TLabel;
    Button3: TButton;
    Label6: TLabel;
    procedure ScrollBar1Change(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

type
  tLEDSetLamp_       = procedure (LED_Nr: tpInteger; Brightness: tpDouble); cdecl;
  tLEDSetPowerlevel_ = procedure (LED_Nr: tpInteger; tppowerLevel: tpDouble); cdecl;
  tLEDGetWavelength_ = procedure (LED_Nr: tpInteger; Output: PChar); cdecl;
  tLEDGetMaxPower_   = procedure (LED_Nr: tpInteger; MaxPower: tpDouble); cdecl;
  tLEDGetAUXin_ =      procedure ( Value: tpDouble); cdecl;

var
  Form1: TForm1;

{$ifdef static}
  procedure _LEDSetLamp_(LED_Nr: tpInteger; Brightness: tpDouble); cdecl; external '../LED.dll';
  procedure _LEDSetPowerlevel_(LED_Nr: tpInteger; tppowerLevel: tpDouble); cdecl; external '../LED.dll';

  procedure _LEDGetWavelength_(LED_Nr: tpInteger; Output: PChar); cdecl; external '../LED.dll';
  procedure _LEDGetMaxPower_ (LED_Nr: tpInteger; MaxPower: tpDouble); cdecl; external '../LED.dll';
  procedure _LEDGetAUXin_ ( Value: tpDouble); cdecl; external '../LED.dll';
{$endif}


implementation

{$R *.dfm}

{$ifndef static}
var
  _LEDSetLamp_ : tLEDSetLamp_;
  _LEDSetPowerlevel_ : tLEDSetPowerlevel_;
  _LEDGetWavelength_ : tLEDGetWavelength_;
  _LEDGetMaxPower_ : tLEDGetMaxPower_;
  _LEDGetAUXin_ : tLEDGetAUXin_;
  DLLHandle : THandle;
{$endif}

procedure TForm1.FormCreate(Sender: TObject);
begin
  {$ifndef static}
  DLLHandle := LoadLibrary(PChar('../LED.dll'));
  if DLLHandle=0 then begin
    ShowMessage('"../LED.dll" not found!');
    halt(0);
  end;

  @_LEDSetPowerlevel_ := GetProcAddress(DLLHandle,'_LEDSetPowerlevel_');
  if @_LEDSetPowerlevel_ = nil then begin
    ShowMessage('"_LEDSetPowerlevel_" in LED.dll not found!');
    halt(0);
  end;

  @_LEDSetLamp_ := GetProcAddress(DLLHandle,'_LEDSetLamp_');
  if @_LEDSetLamp_ = nil then begin
    ShowMessage('"_LEDSetLamp_" in LED.dll not found!');
    halt(0);
  end;


  @_LEDGetWavelength_ := GetProcAddress(DLLHandle, '_LEDGetWavelength_');
  if @_LEDGetWavelength_ = nil then begin
    ShowMessage('"_LEDGetWavelength_" in LED.dll not found!');
    halt(0);
  end;

  @_LEDGetMaxPower_ := GetProcAddress(DLLHandle,'_LEDGetMaxPower_');
  if @_LEDGetMaxPower_ = nil then begin
    ShowMessage('"_LEDGetMaxPower_" in LED.dll not found!');
    halt(0);
  end;

  @_LEDGetAUXin_ := GetProcAddress(DLLHandle,'_LEDGetAUXin_');
  if @_LEDGetAUXin_ = nil then begin
    ShowMessage('"_LEDGetAUXin_" in LED.dll not found!');
    halt(0);
  end;

  Sleep(10); // ???
  {$endif}

  ScrollBar1.Position := 1;
  ScrollBar2.Position := 50;// 50% calls ScrollBar1Change(Sender);

  Label1.Caption:='?';
  Label5.Caption:='?';
  Label6.Caption:='?';
end;

procedure TForm1.ScrollBar1Change(Sender: TObject);
var
  LEDNr : integer;
  Bright : double;
begin
  LEDNr := ScrollBar1.Position;
  Bright := ScrollBar2.Position;
  _LEDSetLamp_(@LEDNr,  @Bright);
end;

//Check Power
procedure TForm1.Button1Click(Sender: TObject);
var
  LEDNr: Integer;
  Brightness, MaxPower: double;
begin
  Brightness := ScrollBar2.Position; // in %
  LEDNr := ScrollBar1.Position;
  _LEDGetMaxPower_ (@LEDNr, @MaxPower);
  Label1.Caption:=FloatToStrF(Brightness/100*MaxPower, ffFixed,7,3)+'mW';
end;

//CheckWavelength
procedure TForm1.Button2Click(Sender: TObject);
var
  sTemp: String;
  LEDNr : integer;
begin
  LEDNr:=ScrollBar1.Position;
  STemp:='someting'+#0;
  _LEDGetWavelength_(@LEDNr, @sTemp[1]);
  Label5.Caption:=string(pchar(sTemp));
end;

procedure TForm1.Button3Click(Sender: TObject);
var Value : double;
begin
  _LEDGetAUXin_ (@Value);
  Label6.Caption:=FloatToStrF(Value, ffFixed, 7, 3)+'V';
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  {$ifndef static}
  FreeLibrary(DLLHandle);
  {$endif}
end;

end.
